/*
 * Infracerveny prijimac pomoci Arduina.
 *
 * K digitalnimu pinu cislo 3 pripojte IR diodu s vlnovou delkou
 * 940nm. K pinu cislo 11 pak vystup IR prijimace.
 *
 * Pote, co naprogramujete Arduino (pomoci tlacitka Upload) spustte
 * "Serial Monitor" (tlacitko v pravem hornim rohu). Pokud namirite
 * dalkove ovladani na IR prijimac a zmacknete libovolne tlacitko,
 * v okne serial monitoru se zobrazi jaka hodnota byla prijata,
 * kolik mela bitu a pokud je ovladac od Sony, pripadne implementuje
 * standard NEC, RC5 nebo RC6, je vyslany kod spravne rozpoznan a je
 * zobrazena jeho hexa hodnota.
 *
 * Kod je inspirovan prikladem od autora knihovny IRremote,
 * Kena Shirriffa http://arcfn.com
 */

// Pouzij knihovnu IRremote
#include <IRremote.h>
// Signal z IR prijimace budeme cist z digitalniho pinu 11.
int RECV_PIN = 11;

// Instance tridy IRrecv.
IRrecv irrecv(RECV_PIN);
// Prijata data se budou ukladat do promenne 'results'.
decode_results results;

/**
 * Tato funkce se provede pouze jednou, po spusteni Arduina.
 * Nastavi komunikaci s PC pres UART o rychlosti 9600 baudu
 * a nastavi IR prijimac.
 */
void setup()
{
  Serial.begin(9600);
  irrecv.enableIRIn();
}

/**
 * Nekonecna smycka. Tato funkce se opakuje
 * neustale dokola.
 */
void loop() {
  // Pokud jsme prijali nejaka data z IR prijimace...
  if (irrecv.decode(&results)) {
    // ...posli na UART (do pripojeneho PC) prijaty kod...
    Serial.println(results.value, HEX);
    // ...a vytiskni dalsi informace.
    dump(&results);
    irrecv.resume(); // Zacni prijimat dalsi data.
  }
}

/**
 * Tato funkce odesila na UART rozhrani dalsi informace
 * o prijatych datech z IR prijimace.
 */
void dump(decode_results *results) {
  // Kolik bitu jsme prijali?
  int count = results->rawlen;
  
  // Zjisti protokol dalkoveho ovladani.
  // Zname protokoly jsou SONY, NEC, RC5 a RC6.
  if (results->decode_type == UNKNOWN) {
    Serial.println("Could not decode message");
  } 
  else {
    if (results->decode_type == NEC) {
      Serial.print("Decoded NEC: ");
    } 
    else if (results->decode_type == SONY) {
      Serial.print("Decoded SONY: ");
    } 
    else if (results->decode_type == RC5) {
      Serial.print("Decoded RC5: ");
    } 
    else if (results->decode_type == RC6) {
      Serial.print("Decoded RC6: ");
    }
    
    // Pokud jsme prijali data ve znamem protokolu,
    // dekoduj je  zobraz pocet prijatych bitu a jejich
    // hexa hodnotu.
    Serial.print(results->value, HEX);
    Serial.print(" (");
    Serial.print(results->bits, DEC);
    Serial.println(" bits)");
  }
  
  // Vytiskni prijata data v nedekodovane podobe.
  Serial.print("Raw (");
  Serial.print(count, DEC);
  Serial.print("): ");

  for (int i = 0; i < count; i++) {
    if ((i % 2) == 1) {
      Serial.print(results->rawbuf[i]*USECPERTICK, DEC);
    } 
    else {
      Serial.print(-(int)results->rawbuf[i]*USECPERTICK, DEC);
    }
    Serial.print(" ");
  }
  Serial.println("");
}
