/*
 * Infracerveny vysilac pomoci Arduina.
 *
 * K digitalnimu pinu cislo 3 pripojte IR diodu s vlnovou delkou
 * 940nm. K pinu cislo 11 pak vystup IR prijimace.
 *
 * Tento kod funguje pouze pro zarizeni Sony STR-DE495P. Toto zarizeni
 * prijima kody o delce 15 bitu. My budeme odesilat pouze dva prikazy --
 * pro zesileni a zeslabeni zvuku.
 *
 * Pro jina zarizeni je potreba kod upravit. Podivejte se do funkce loop()
 * a upravte volani funkce sendIR(). Prvni argument funkce je cislo kodu
 * a druhy je pocet odesilanych bitu.
 *
 * Pote, co naprogramujete Arduino (pomoci tlacitka Upload) spustte
 * "Serial Monitor" (tlacitko v pravem hornim rohu). Napiste znak "+" nebo
 * "-" a enterem odeslete do Ardunia. To vysle 3x za sebou pozadovany
 * kod. 
 *
 * Kod je inspirovan prikladem od autora knihovny IRremote,
 * Kena Shirriffa http://arcfn.com
 */

// Pouzij knihovnu IRremote
#include <IRremote.h>

// Instance tridy IRsend.
IRsend irsend;

/**
 * Tato funkce se provede pouze jednou, po spusteni Arduina.
 * Nastavi komunikaci s PC pres UART o rychlosti 9600 baudu
 * a nastavi IR prijimac.
 */
void setup() {
  Serial.begin(9600);
}

/**
 * Nekonecna smycka. Tato funkce se opakuje
 * neustale dokola.
 */
void loop() {
  // Cekej, az na rozhrani UART (z pripojeneho PC) prijdou nejaka data.
  if(Serial.available() > 0) {
    // Precti jeden bajt z UART.
    int command = Serial.read();
    // A podle toho, co jsme precetli se rozhodni.
    switch(command) {
    // Precetli jsme znak plus, vysli IR signal pro zesileni
    // zvuku. Kod je 0x240C a ma delku 15 bitu.
    case '+':
      sendIR(0x240C, 15);
      Serial.println("Zesiluji.");
      break;
    // Precetli jsme znak minus, vysli IR signal pro zeslabeni
    // zvuku. Kod je 0x640C a ma delku 15 bitu.
    case '-':
      sendIR(0x640C, 15);
      Serial.println("Zeslabuji.");
      break;
    // Precetli jsme nejaky jiny znak. Chyba.
    default:
      Serial.println("Neznamy prikaz!");
    }
  }
}

/**
 * Funkce odesle 'data' o delce 'nbits' bitu na pin
 * cislo 3 na kterem je pripojena IR dioda.
 */
void sendIR(unsigned long data, int nbits) {
    // IR protokol ocekava opakovani jednoho kodu 3x
    // za sebou s malym spozdenim.
    for (int i = 0; i < 3; i++) {
      irsend.sendSony(data, nbits);
      delay(10);
    }
}

